// emacs: this is -*- c++ -*-
//
//   @file    imapCache.h        
//            
//            interface to a convenient cache type, allows any number of 
//            different values of observable for any give set of 
//            x1, x2, Q2, order
//  
//   Copyright (C) 2016 M.Sutton (sutt@cern.ch)    
//
//   $Id: imapCache.h, v0.0   Thu 14 Jul 2016 13:21:43 CEST sutt $


#ifndef  IMAPCACHE_H
#define  IMAPCACHE_H

#include <iostream>
#include <vector>
#include <map>


extern "C" int currentcacheid_();

template<typename T>
class imapCache : public T {

public:
  
  /// convenient typedefs 
  
  //  typedef std::map<double, std::vector<double> >                            map_t; 
  //  typedef std::map<double, std::vector<double> >::iterator               iterator; 
  //  typedef std::map<double, std::vector<double> >::const_iterator   const_iterator; 

  //  typename T                            map_t; 
  //  typename T::iterator               iterator; 
  //  typedef typename T::const_iterator   const_iterator; 

public:

  imapCache( int nproc=0 ) : 
    mx1(0),  mx2(0),  mQ2(0), 
    morder(0),
    mnproc(nproc),
    mfilled(false)
  { } 
  
  virtual ~imapCache() { } 
  
  bool filled() const { return mfilled; }

  void setnproc( int nproc ) { mnproc = nproc; }

  /// "vector-like" operators

  //  size_t size() const { return this->size(); }
  //  void clear()        { this->clear(); }

  /// iterators
  //  iterator begin() { return this->begin(); }
  //  iterator end()   { return this->end(); }

  //  const_iterator begin() const { return this->begin(); }
  //  const_iterator end()   const { return this->end(); }

  

  /// find the weight vector corresponding to this observable value
  /// or else add a new one to this cache

  virtual std::vector<double>& weights( double obs ) = 0;

  /// have any of the kinematic variables changed ?
  
  virtual bool changed( double obs, int order, double x1, double x2, double Q2 ) = 0;


  /// update the cached kinematics
    
  virtual void update( double obs, int order, double x1, double x2, double Q2 ) = 0;
  
public:

  double  mx1,  mx2, mQ2;
  int     morder;
  int     mnproc;		    

  bool    mfilled;

};

template<typename T>
inline std::ostream& operator<<( std::ostream& s, const imapCache<T>& _p ) { 
  return s;
}


#endif  // IMAPCACHE_H 










